@@ -20,7 +20,7 @@ try:
     import dbusmock
 except ImportError:
     sys.stderr.write('You need python-dbusmock (http://pypi.python.org/pypi/python-dbusmock) for this test suite.\n')
-    sys.exit(0)
+    sys.exit(1)
 
 try:
     from gi.repository import Gio
@@ -54,13 +54,11 @@ class GSDTestCase(dbusmock.DBusTestCase):
     @classmethod
     def setUpClass(klass):
         os.environ['GIO_USE_VFS'] = 'local'
+        os.environ['GVFS_DISABLE_FUSE'] = '1'
         # we do some string checks, disable translations
         os.environ['LC_MESSAGES'] = 'C'
         klass.workdir = tempfile.mkdtemp(prefix='gsd-power-test')
 
-        # start X.org server with dummy driver; this is needed until Xvfb
-        # supports XRandR:
-        # http://lists.x.org/archives/xorg-devel/2013-January/035114.html
         klass.start_xorg()
 
         # tell dconf and friends to use our config/runtime directories
@@ -70,6 +68,7 @@ class GSDTestCase(dbusmock.DBusTestCase):
 
         # work around https://bugzilla.gnome.org/show_bug.cgi?id=689136
         os.makedirs(os.path.join(os.environ['XDG_CONFIG_HOME'], 'dconf'))
+        os.makedirs(os.environ['XDG_RUNTIME_DIR'], mode=0o700)
 
         klass.start_system_bus()
         klass.start_session_bus()
@@ -151,26 +150,12 @@ class GSDTestCase(dbusmock.DBusTestCase):
         klass.monitor_log.flush()
         klass.monitor_log.close()
 
-    def start_logind(self):
+    def start_logind(self, parameters=None):
         '''start mock logind'''
 
-        self.logind = self.spawn_server('org.freedesktop.login1',
-                                        '/org/freedesktop/login1',
-                                        'org.freedesktop.login1.Manager',
-                                        system_bus=True,
-                                        stdout=subprocess.PIPE)
-        self.obj_logind = self.system_bus_con.get_object(
-            'org.freedesktop.login1', '/org/freedesktop/login1')
-
-        self.obj_logind.AddMethods('',
-            [
-                ('PowerOff', 'b', '', ''),
-                ('Suspend', 'b', '', ''),
-                ('Hibernate', 'b', '', ''),
-                ('Inhibit', 'ssss', 'h', 'ret = 5'),
-                ('CanSuspend', '', 's', "ret = 'yes'"),
-                ('CanHibernate', '', 's', "ret = 'no'")
-            ], dbus_interface='org.freedesktop.DBus.Mock')
+        self.logind, self.logind_obj = self.spawn_server_template('logind',
+                                                                  parameters or {},
+                                                                  stdout=subprocess.PIPE)
 
         # set log to nonblocking
         set_nonblock(self.logind.stdout)
@@ -185,7 +170,9 @@ class GSDTestCase(dbusmock.DBusTestCase):
         ''' start mutter '''
 
         klass.mutter_log = open(os.path.join(klass.workdir, 'mutter.log'), 'wb')
-        klass.mutter = subprocess.Popen(['mutter'],
+        # See https://gitlab.gnome.org/GNOME/mutter/merge_requests/15
+        os.environ['XDG_SESSION_TYPE'] = 'x11'
+        klass.mutter = subprocess.Popen(['mutter', '--x11'],
                                          stdout=klass.monitor_log,
                                          stderr=subprocess.STDOUT)
 
@@ -201,21 +188,12 @@ class GSDTestCase(dbusmock.DBusTestCase):
 
     @classmethod
     def start_xorg(klass):
-        '''start X.org server with dummy driver'''
+        '''start Xvfb server'''
 
-        conf = os.path.join(os.path.dirname(__file__), 'xorg-dummy.conf')
-
-        # some distros like Fedora install Xorg as suid root; copy it into our
-        # workdir to drop the suid bit and run it as user
-        if GLib.file_test('/usr/libexec/Xorg.bin', GLib.FileTest.IS_EXECUTABLE):
-            xorg = '/usr/libexec/Xorg.bin'
-        else:
-            out = GLib.find_program_in_path ('Xorg')
-            if not out:
-                sys.stderr.write('ERROR: Xorg not installed\n')
-                sys.exit(1)
-            xorg = os.path.join(klass.workdir, 'Xorg')
-            shutil.copy(out, xorg)
+        xorg = GLib.find_program_in_path ('Xvfb')
+        if not xorg:
+            sys.stderr.write('Cannot start X.org, Xvfb binary not found\n')
+            sys.exit(1)
 
         display_num = 99
 
@@ -223,10 +201,8 @@ class GSDTestCase(dbusmock.DBusTestCase):
             sys.stderr.write('Cannot start X.org, an instance already exists\n')
             sys.exit(1)
 
-        # You can rename the log file to *.log if you want to see it on test
-        # case failures
-        log = os.path.join(klass.workdir, 'Xorg.out')
-        klass.xorg = subprocess.Popen([xorg, '-config', conf, '-logfile', log, ':%d' % display_num],
+        # Composite extension won't load unless at least 24bpp is set
+        klass.xorg = subprocess.Popen([xorg, ':%d' % display_num, "-screen", "0", "1280x1024x24", "+extension", "GLX"],
                                       stderr=subprocess.PIPE)
         os.environ['DISPLAY'] = ':%d' % display_num
 
@@ -243,8 +219,7 @@ class GSDTestCase(dbusmock.DBusTestCase):
                                stderr=subprocess.PIPE) == 0:
                 break
         if timeout <= 0:
-            with open(log) as f:
-                sys.stderr.write('Cannot start X.org with dummy driver.  Log:\n%s\n--------' % f.read())
+            sys.stderr.write('Cannot start Xvfb.\n--------')
             sys.exit(1)
 
     @classmethod
